## 总体说明

该文档用于说明所有说明性文档用途及源码的存放组织结构，以下分别就文档和具体源码做具体说明：

## 文档

所有文档存放在`doc`目录下，主要分为与平台无关的业务设计说明文档，以及与具体平台相关部署流程文档及开发体验文档：

### 业务设计说明文档

- **数据结构设计文档：该文档用于总体说明合约编写的目的、背景以及设计思路等，并具体介绍了源码设计规范及具体的数据结构，为整个合约设计的核心文档，若要了解合约设计建议从该文档开始。**
- 链码设计文档：用于介绍各个合约、合约之间的调用关系，以及合约对外暴露的所有接口（方法）的行为

### 平台相关文档

- 各框架合约编写体会：用于记录不同框架下编写合约的区别和体会
- Fabric合约部署流程：从0开始介绍在Fabric平台下部署相关智能合约，并介绍合约调用的基本流程
- XuperChain合约部署流程：从0开始介绍在XuperChain平台下部署相关智能合约，并介绍合约调用的基本流程
- CITA合约部署流程：从0开始介绍在CITA平台下部署相关智能合约，并介绍合约调用的基本流程
- FISCO BCOS合约部署流程：从0开始介绍在FISCO BCOS平台下部署相关智能合约，并介绍合约调用的基本流程

**注**：为了方便阅读针对每一个文档提供了PDF和Markdown两种格式，两者内容完全相同可以根据实际情况选择。

## 代码

目前Fabric、XuperChain平台上的智能合约通过go语言实现，而CITA、FISCO BCOS平台上的智能合约基于solidity语言0.4.24版本实现。以下分别以go语言及solidity语言出发，分别介绍合约在各个平台上的代码组织结构：

### go语言

go语言相关代码都存储在`src`目录下，主要涉及以下子目录：

- core：存放与核心业务相关的代码
- oracles：存放与预言机服务相关的合约代码
- fabric：存放用于编译Fabric区块链上智能合约的代码
- xchain：存放用于编译XuperChain区块链上智能合约的代码

以下对每一个子目录分别介绍：

#### core

用于存放核心业务逻辑，涉及的子目录有：

- common：存放基础通用逻辑
- contract：存放通用的合约定义逻辑
- invokes：用于合约调用入口封装
- orgnization：用于定义合约涉及到的人员定义
- project：存放与一个业务方向相关的代码
- record：存放通用的存储相关逻辑
- rules：存放规则定义、调用与验证等逻辑

#### oracles

用于存放与预言机服务相关逻辑，该目录下每一个子目录对应一个具体的预言机服务，目前支持的预言机服务有：

- face：人脸识别服务
- identify：物体识别服务
- location：定位服务
- time：时间服务

每一个预言机服务的子目录下主要涉及以下内容：

- invokes：用于与该预言机相关的合约调用入口封装
- service：存放与预言机服务相关的具体代码逻辑
- dummy：为service目录下的子目录，由于目前未配置具体预言机服务，这里在智能合约中模拟预言机服务的验证过程

#### fabric

存放与fabric平台相关的代码，主要涉及的子目录如下：

- contract：为了剥离平台无关代码，我们在`contract/contract`目录下定义了统一的智能合约抽象接口，该子目录为抽象接口的具体实现
- audit：存放核心（审计）智能合约的入口
- face：存放人脸识别智能合约的入口
- identify：存放物体识别智能合约的入口
- location：存放定位智能合约的入口
- time：存放时间智能合约的入口

fabric目录下有一个`Makefile`文件用于编译fabric平台下所有合约的编译测试

#### xchain

存放与XuperChain平台相关的代码，主要涉及的子目录如下：

- contract：为了剥离平台无关代码，我们在`contract/contract`目录下定义了统一的智能合约抽象接口，该子目录为抽象接口的具体实现
- audit：存放核心（审计）智能合约的入口
- face：存放人脸识别智能合约的入口
- identify：存放物体识别智能合约的入口
- location：存放定位智能合约的入口
- time：存放时间智能合约的入口

xchain目录下会有一个`Makefile`文件用于编译XuperChain平台下的所有智能合约

### solidity语言

solidity语言存放在`src/solidity`子目录下，主要包括用于存放审计核心业务逻辑的audit和预言机服务相关合约目录oracles：

#### audit

audit目录下主要包含如下文件：

- interface目录：用于存放预言机服务合约共同实现的接口
- audit.sol文件：审计合约定义文件
- rules.sol文件：存放与规则相关的逻辑

#### oracles

oracles目录下主要包含如下目录：

- face：存放人脸识别预言机相关的智能合约代码
- identify：存放物体识别预言机相关的智能合约代码
- location：存放定位服务预言机相关的智能合约代码
- time：存放时间服务预言机相关的智能合约代码

## 注释说明

由于本次比赛的受众为国内的评委及选手主要来自国内，为了具有更好的阅读体验建议所有的注释都以中文形式添加，目前有两种类型的注释：

- 以`fixme`开头的是我们在本次比赛不会实现的部分，一般涉及到预言机服务相关的交互逻辑
- 上述类型注释之外为普通注释，为了解释说明一些关键逻辑
