## 前言

该文档侧重描述合约的编写体验，由于Fabric、XuperChain支持高级语言开发（共通语言为go），而CITA、FISCO BCOS虽然也支持native合约，但主要合约开发语言还是solidity，因此该文档会首先从编写代码的角度分析go语言及solidity的开发区别及体会。

此外，合约编写完成后的测试、部署也是开发合约过程中很重要的一环，因此该文档也会描述不同平台上测试、部署体验。

最后我们会就不同平台的社区生态对开发者的支持友好程度给出个人的见解。

## 编写体验

### go语言

就语言本身来说，go语言延续了c语言简洁易上手的风格，并未采用目前主流的面向对象编程范式，而是发明了面向链接的编程范式，充分体现了“组合优于继承”的编程思想，加上丰富的标准库，以及对并发、测试和垃圾回收的很好支持，使得go语言开发成本较低。但go语言也存在错误机制臃肿、缺少对泛型支持等不足，但不存在明显短板，个人认为比较适合做联盟链上的合约开发语言。

开发环境方面，我们go语言开发主要基于IntelliJ的go语言IDE——Goland进行开发，该IDE对go语言具有很好的支持，包括：智能输入提示及补全、便利的重构操作、查找所有应用、语法错误提示、自动实现接口等，IDE提供了debug工具为编码后的测试工作提供了便利。

### solidity语言

solidity语言为智能合约而生，内置了很多与智能合约及链上业务强相关的语法（如payable、storage关键字，大整数类型等），采用了面向对象编程范式，编程风格与大家普遍熟悉的Java、C#类似，上手门槛较低。但由于发展时间较短等因素导致该语言成熟度较低，当应用到联盟链这种业务场景较多的情况时，程序表达能力略显不足。

开发环境方面，我们主要基于VS Code配合solidity插件（开发者为Juan Blanco的版本）进行代码编写，该插件支持语法高亮、跳转，以及语法错误检测功能，配置solidity可执行路径后也可以对合约代码编译，但智能补全、查找引用等功能支持不太好。由于上述插件不支持调试部署，当开发完成后我们会基于太坊官方提供的网页版IDE——remix ide进行调试和预部署的测试工作。通过上述两个IDE互相补充短板，能够较为顺利地帮助我们完成合约的开发工作。

## 测试部署体验

### Fabric

Fabric平台链可通过docker、docker-compose部署，再配合shell脚本可以极大地简化部署流程，在加上docker容器的平台无关性为测试部署操作提供了便利。go语言内置对单元测试的支持，可以通过单元测试覆盖、验证核心或者复杂的业务逻辑。peer节点自带命令行模式，chaincode子命令提供了对链码的日常维护以及查询工作，能够满足平时的测试部署需求。

不足之处在于MSP管理、环境变量配置较为复杂，官方提供了一些命令行工具，但仍需要人工处理很多琐碎的配置工作，这或许是Fabric的主推方IBM公司为自家IBM云服务售卖（在IBM云服务中实操Fabric个人觉得体验不错）铺路。

### XuperChain

XuperChain单节点模式部署简单，且提供了公开的测试环境，能够让开发者无须感知链本身而进行智能合约方面的开发工作。如Fabric类似，使用go语言提高了方法级的测试便捷性。

不足之处在于对于Linux之外的操作系统部署体验不友好，本人的Mac OSX系统可以成功编译但部署时会导致链奔溃，经过多次的尝试之后最终放弃转而使用Ubuntu才操作成功。

自带Asset（资产）属性对于合约开发而言是一把双刃剑，好处在于能够减轻智能合约中资产逻辑处理的负担，且能够有效控制合约运算资源防止合约中的死循环等；坏处在于在测试过程中必须要建立专门的合约地址，并且向该合约地址转入足够的gas才能够运转，这对于合约测试而言增加了操作步骤。

#### FISCO BCOS

FISCO BCOS提供了沉浸式控制台，支持命令行completion，合约部署的CNS模式结合智能合约版本能够让开发者几乎无感知链的情况下运维智能合约，为合约的部署测试提供了极大的便利性。

但由于solidity语言对单元测试不友好，很多细节的测试操作也需要实际上链测试；以及控制台有时会感知不到合约文件的更新，需要频繁重启控制台比较影响开发体验。

### CITA

CITA 完全兼容 solidity 语言，而且提供了 remix ide。让 ethereum 社区开发者节省很多时间。其提供的 docker 容器让部署变得非常简单。 test net 及水龙头支持也很完善。

我个人在使用 cita-cli 工具时，发现其无法针对 `string[]` 类型的参数进行 encode abi 操作。不过问题在 remix ide 上得到了解决。

## 社区友好度

### Fabric

Fabric的开发文档覆盖全面，重要的技术点都会有相关的示例文档，不足之处在于中文翻译不全，这比较影响非英语母语受的众阅读效率。

由于起步较早，Fabric社区已趋于成熟，互联网上存在很多中英文的教程、专栏介绍，平台中遇到的很多问题都可以通过github issue、stackoverflow、搜索引擎等多个渠道中找到。但由于Fabric各版本变化较大，于是出现了与版本相关的技术总结及问题分析文章，这比较影响学习及问题定位。

### XuperChain

XuperChain旗下的[区块链大学](https://xuperchain.baidu.com/n/college)会一些与区块链相关的技术干货分享，这对于区块链及XuperChain学习很有帮助。XuperChain具有全中文的平台说明文档，原理性说明文档很出彩，常见使用场景都会有使用示例覆盖，希望能够补充命令行指令文档。

### FISCO BCOS

FISCO BCOS也有全中文的平台说明文档，其中使用手册部分能够将原理与具体操作结合深入浅出地为开发者讲解相关知识，对开发者使用体验很友好。“FSICO BCOS开源社区”微信公众号分享内容很多，技术干货很足，不过鉴于开发者一般都会在电脑上阅读源码，希望这些分享内容也能够以网站的形式体现。

### CITA

CITA 拥有国内一流的社区生态。几十家入驻 citahub 的企业可以见其深受国内区块链行业公司的青睐。详细的文档，国密算法的支持，丰富的周边配套设施，周到贴心的 BD 人员让人可以相信 cita 社区会拥有更好的未来。也希望 cita 社区在国外发力，争取更多国际上的区块链从业者加入进来。

